@@ -1,4 +1,4 @@
-#' @title Read Ilostat Table of Contents
+#' @title Read ilostat Table of Contents
 #' @description Download one table of contents from ilostat \url{https://ilostat.ilo.org} via bulk download facility 
 #' \url{https://ilostat.ilo.org/data/bulk/}.
 #' @param segment A character, way to get datasets by: \code{"indicator"} (default) or \code{"ref_area"}, 
@@ -15,6 +15,8 @@
 #'			filters detect on variables.
 #' @param fixed a logical, if \code{TRUE} (default), pattern is a string to be matched as is,
 #'        Change to \code{FALSE} if more complex regex matching is needed.
+#' @param quiet a logical, if \code{FALSE} , return message from processing, \code{TRUE} (default).
+#' 			Can be set also with options(ilostat_quiet = TRUE), 
 #' @return A tibble with ten columns depending of the segment: indicator or ref_area
 #' 		\itemize{
 #'      \item{\code{id}} : The codename of dataset of theme, will be used by the get_ilostat and get_ilostat_raw functions,
@@ -29,13 +31,13 @@
 #'      \item{\code{...}} : Others relevant information
 #'      }
 #' @seealso \code{\link{get_ilostat}}.
-#' @details The TOC in English by indicator is downloaded from \url{https://webapps.ilo.org/ilostat-files/WEB_bulk_download/indicator/table_of_contents_en.csv}. 
+#' @details The toc in English by indicator is downloaded from ilostat API \url{https://rplumber.ilo.org/__docs__/#get-/metadata/toc/indicator/}. 
 #' The values in column 'id' should be used to download a selected dataset.
-#' @details The TOC in English by ref_area is downloaded from \url{https://webapps.ilo.org/ilostat-files/WEB_bulk_download/ref_area/table_of_contents_en.csv}. 
+#' @details The toc in English by ref_area is downloaded from ilostat API \url{https://rplumber.ilo.org/__docs__/#get-/metadata/toc/ref_area/}. 
 #' The values in column 'id' should be used to download a selected dataset.
 #' @references
 #' See citation("Rilostat")
-#' ilostat bulk download facility user guidelines \url{https://webapps.ilo.org/ilostat-files/WEB_bulk_download/ILOSTAT_BulkDownload_Guidelines.pdf} 
+#' ilostat bulk download facility user guidelines \url{https://ilostat.ilo.org/data/bulk/} 
 #' @author David Bescond \email{bescond@ilo.org}
 #' @keywords utilities database
 #' @examples
@@ -62,20 +64,21 @@
 #'  head(toc)
 #'  toc <- get_ilostat_toc(search = 'youth|adult', fixed = FALSE)
 #'  head(toc)
-#'  toc <- get_ilostat_toc(search = c('youth','adult'), fixed = FALSE)
-#'  head(toc)
 #' ##
 #' }
 #' @export
 
+
+
 get_ilostat_toc <- function(segment = getOption('ilostat_segment', 'indicator'), 
 							lang = getOption('ilostat_lang', 'en'), 
 							search = getOption('ilostat_search', 'none'), 
 							filters = getOption('ilostat_filter', 'none'),
-							fixed = getOption('ilostat_fixed', TRUE)) {
+							fixed = getOption('ilostat_fixed', TRUE), 
+							quiet = getOption('ilostat_quiet', FALSE)) {
   
   
-  set_ilostat_toc(segment, lang)
+  set_ilostat_toc(segment, lang, quiet)
   
   y <- get(paste0(".ilostatTOC", segment, lang), envir = .ilostatEnv) 
   
@@ -117,32 +120,39 @@ get_ilostat_toc <- function(segment = getOption('ilostat_segment', 'indicator'),
     }
   }	
    
-  invisible(gc(reset = TRUE))
    
   y
 }
 
 
-set_ilostat_toc <- function(segment, lang) {
-  
-  if (!exists(paste0(".ilostatTOC", segment, lang), envir = .ilostatEnv)) {
-    
-	# base <- ilostat_url()
-    
-	url <- paste0(ilostat_url() , segment, "/table_of_contents_",lang,".rds")
-    
-	.ilostatTOC <- read_rds(
-	
-						url 
-	
-						) # %>% as_tibble %>% mutate_if(is.factor, as.character)
-    
-	assign(paste0(".ilostatTOC", segment, lang), .ilostatTOC, envir = .ilostatEnv)
-  
+set_ilostat_toc <- function(segment, lang, quiet) {
+  toc_name <- paste0(".ilostatTOC", segment, lang)
+
+  if (!exists(toc_name, envir = .ilostatEnv)) {
+    url <- paste0(ilostat_url(), segment, "/table_of_contents_", lang, ".rds")
+    if(!quiet) message("Trying toc URL '", url, "'")
+
+    # Build User-Agent
+    ua <- build_user_agent()
+
+    # Perform request
+    resp <- request(url) |>
+      req_headers(`User-Agent` = ua) |>
+      req_perform()
+
+    if (resp_status(resp) != 200) {
+      stop("Table of contents for segment '", segment, "' and lang '", lang, "' not available")
+    }
+
+    # Save and read toc
+    tmpfile <- tempfile(fileext = ".rds")
+    writeBin(resp_body_raw(resp), tmpfile)
+
+    .ilostatTOC <- read_rds(tmpfile)
+
+    # Cache in environment
+    assign(toc_name, .ilostatTOC, envir = .ilostatEnv)
   }
-  
+
   invisible(0)
-
-  }
-
-
+}
